@@ -11,20 +11,41 @@
 #include <QDebug>
 #include <QMessageBox>
 
+class ConfigWindowPrivate
+{
+public:
+    ConfigWindowPrivate(ConfigWindow * p) :
+            gconfUtility(new GConfUtility(p)),
+            dbusUtility(new DbusUtility(p)),
+            parent(p)
+    {
+
+    }
+
+    ~ConfigWindowPrivate()
+    {
+        qDebug() << "VICaR: ConfigWindow Destructing";
+    }
+
+    GConfUtility * gconfUtility;
+    DbusUtility * dbusUtility;
+    ConfigWindow * const parent;
+};
+
+// ---------------------------------------------------------------------------
+
 ConfigWindow::ConfigWindow(QWidget *parent) :
     QMainWindow(parent),
-    ui(new Ui::ConfigWindow)
+    ui(new Ui::ConfigWindow),
+    d(new ConfigWindowPrivate(this))
 {
     ui->setupUi(this);
-    gconfUtility = new GConfUtility();
     loadValues();
 }
 
 ConfigWindow::~ConfigWindow()
 {
     delete ui;
-    delete gconfUtility;
-    gconfUtility = 0;
 }
 
 void ConfigWindow::changeEvent(QEvent *e)
@@ -70,19 +91,18 @@ void ConfigWindow::on_actionReset_triggered()
     ui->lineEditDTMFSuffix->clear();
     //ui->spinBoxDTMFDelay->setValue(1);
     qDebug() << "Settings reset";
-    DbusUtility dbusUtility = DbusUtility();
-    dbusUtility.displayNotification("VICaR: Settings reset.");
+    d->dbusUtility->displayNotification("VICaR: Settings reset.");
 }
 
 void ConfigWindow::loadValues(){
 
-    bool isRoutingEnabled = gconfUtility->getGconfValueBoolean("routing_enabled");
-    QString strCallingCardNumber = gconfUtility->getGconfValueString("calling_card_number");
-    QString strNumbersToExclude = gconfUtility->getGconfValueString("numbers_to_exclude");
-    QString strDTMFFormat = gconfUtility->getGconfValueString("dtmf_format");
-    QString strDTMFPrefix = gconfUtility->getGconfValueString("dtmf_prefix");
-    QString strDTMFSuffix = gconfUtility->getGconfValueString("dtmf_suffix");
-    //int intDTMFDelay = gconfUtility->getGconfValueInteger("dtmf_delay");
+    bool isRoutingEnabled = d->gconfUtility->getGconfValueBoolean("routing_enabled");
+    QString strCallingCardNumber = d->gconfUtility->getGconfValueString("calling_card_number");
+    QString strNumbersToExclude = d->gconfUtility->getGconfValueString("numbers_to_exclude");
+    QString strDTMFFormat = d->gconfUtility->getGconfValueString("dtmf_format");
+    QString strDTMFPrefix = d->gconfUtility->getGconfValueString("dtmf_prefix");
+    QString strDTMFSuffix = d->gconfUtility->getGconfValueString("dtmf_suffix");
+    //int intDTMFDelay = d->gconfUtility->getGconfValueInteger("dtmf_delay");
 
     ui->checkBoxIntlCallRedirEnabled->setChecked(isRoutingEnabled);
     ui->lineEditCallingCardNumber->setText(strCallingCardNumber);
@@ -146,9 +166,9 @@ void ConfigWindow::saveConfigData(){
        // int intDTMFDelay = ui->spinBoxDTMFDelay->value();
 
 
-        gconfUtility->setGconfValueBoolean("routing_enabled",isRoutingEnabled);
+        d->gconfUtility->setGconfValueBoolean("routing_enabled",isRoutingEnabled);
 
-        gconfUtility->setGconfValueString("calling_card_number",strCallingCardNumber);
+        d->gconfUtility->setGconfValueString("calling_card_number",strCallingCardNumber);
 
         if (!strNumbersToExclude.isEmpty()){
             strNumbersToExclude = strNumbersToExclude.remove(" ");
@@ -156,24 +176,14 @@ void ConfigWindow::saveConfigData(){
             strNumbersToExclude = strNumbersToExclude.replace("\n",",");
         }
 
-        gconfUtility->setGconfValueString("numbers_to_exclude",strNumbersToExclude);
-        //gconfUtility->setGconfValueInteger("dtmf_delay",intDTMFDelay);
-        gconfUtility->setGconfValueString("dtmf_format",strDTMFFormat);
-        gconfUtility->setGconfValueString("dtmf_prefix",strDTMFPrefix.trimmed());
-        gconfUtility->setGconfValueString("dtmf_suffix",strDTMFSuffix);
+        d->gconfUtility->setGconfValueString("numbers_to_exclude",strNumbersToExclude);
+        //d->gconfUtility->setGconfValueInteger("dtmf_delay",intDTMFDelay);
+        d->gconfUtility->setGconfValueString("dtmf_format",strDTMFFormat);
+        d->gconfUtility->setGconfValueString("dtmf_prefix",strDTMFPrefix.trimmed());
+        d->gconfUtility->setGconfValueString("dtmf_suffix",strDTMFSuffix);
 
-        DbusUtility dbusUtility = DbusUtility();
 
         qDebug() << "Settings updated";
-        dbusUtility.displayNotification("VICaR: Settings updated.");
-
-        if (isRoutingEnabled){
-            qDebug() << "Enable call routing immediately";
-            dbusUtility.sendSignal(APPLICATION_DBUS_PATH,APPLICATION_DBUS_INTERFACE,"startOutgoingCallMonitor");
-        }
-        else{
-            qDebug() << "Disable call routing immediately";
-            dbusUtility.sendSignal(APPLICATION_DBUS_PATH,APPLICATION_DBUS_INTERFACE,"stopOutgoingCallMonitor");
-        }
+        d->dbusUtility->displayNotification("VICaR: Settings updated.");
 }
 
