@@ -12,243 +12,87 @@
 #include <QRegExp>
 #include <QDBusConnection>
 #include <QDBusMessage>
-#include <QTimer>
 #include <QStringListIterator>
 
-//Create a statis Dbus utility object that will be shared across all member functions
-static DbusUtility dbusUtility = DbusUtility();
 static QString strLastDialedNumber = QString();
 
-CallRouter::CallRouter(QObject *parent) :
-    QObject(parent)
+class CallRouterPrivate
 {
-        dbusAdaptor = new VicarDbusAdaptor(this);
-        gconfUtility = new GConfUtility();
+public:
+    CallRouterPrivate(CallRouter * p) :
+        dbusAdaptor(new VicarDbusAdaptor(p)),
+        gconfUtility(new GConfUtility(p)),
+        dbusUtility(new DbusUtility(p)),
+        parent(p)
+    {
+        Q_ASSERT(0 != dbusAdaptor);
+    }
+
+    ~CallRouterPrivate()
+    {
+        qDebug() << "VICaR: Call Router Destructing";
+    }
+
+    VicarDbusAdaptor * dbusAdaptor;
+    GConfUtility * gconfUtility;
+    DbusUtility * dbusUtility;
+    CallRouter * const parent;
+};
+
+// ---------------------------------------------------------------------------
+
+CallRouter::CallRouter(QObject *parent) :
+    QObject(parent),
+    d(new CallRouterPrivate(this))
+{
+        Q_ASSERT(0 != d);
         this->registerDBusService();
         qDebug() << "Registered DBus Service " << APPLICATION_DBUS_SERVICE;
-
-        //If International call routing is already enabled, start monitoring calls
-        //Otherwise wait for the signal from the configuration app
-        if (gconfUtility->getGconfValueBoolean((char *)"routing_enabled")){
-            qDebug() << "Call routing enabled. Starting call monitor..";
-            this->startOutgoingCallMonitor();
-        }
 }
 
 CallRouter::~CallRouter(){
-        delete gconfUtility;
-        gconfUtility = 0;
-
-        delete dbusAdaptor;
-        dbusAdaptor = 0;
 }
 
 void CallRouter::registerDBusService(){
-    QDBusConnection connection = dbusUtility.getConnection();
+    QDBusConnection connection = d->dbusUtility->getConnection();
 
-    if (!connection.registerService(APPLICATION_DBUS_SERVICE)) {
-        qDebug() << dbusUtility.getErrorMessage();
-        exit(1);
+    if (!connection.interface()->isServiceRegistered(APPLICATION_DBUS_SERVICE)){
+
+        if (!connection.registerService(APPLICATION_DBUS_SERVICE)) {
+            qDebug() << d->dbusUtility->getErrorMessage();
+            exit(1);
+        }
     }
 
     if (!connection.registerObject(APPLICATION_DBUS_PATH, this, QDBusConnection::ExportAdaptors)) {
-        qDebug() << dbusUtility.getErrorMessage();
+        qDebug() << d->dbusUtility->getErrorMessage();
         exit(2);
     }
 
-    this->connectToDBusSignals();
+    //this->connectToDBusSignals();
 
 }
 
 
 void CallRouter::unregisterDBusService(){
 
-    this->disconnectFromDBusSignals();
+    //this->disconnectFromDBusSignals();
 
-    QDBusConnection connection = dbusUtility.getConnection();
+    QDBusConnection connection = d->dbusUtility->getConnection();
 
     connection.unregisterObject(APPLICATION_DBUS_PATH,QDBusConnection::UnregisterTree);
 
     if (!connection.unregisterService(APPLICATION_DBUS_SERVICE)) {
-        qDebug() << dbusUtility.getErrorMessage();
+        qDebug() << d->dbusUtility->getErrorMessage();
         exit(3);
     }
 
 }
 
-void CallRouter::connectToDBusSignals(){
-
-    QDBusConnection connection = dbusUtility.getConnection();
-
-    // Connect to the signal to enable call routing
-    bool success = connection.connect(QString(""),QString(""),
-                       APPLICATION_DBUS_INTERFACE,
-                       QString("startOutgoingCallMonitor"),this,
-                       SLOT(startOutgoingCallMonitor()));
-
-    if (success){
-        qDebug() << "Successfully connected to Dbus signal org.maemo.vicar.startOutgoingCallMonitor";
-    }
-    else{
-        qDebug() << "Failed to connect to Dbus signal org.maemo.vicar.startOutgoingCallMonitor";
-        qDebug() <<"DBus Error: "<< qPrintable(dbusUtility.getErrorMessage());
-    }
-
-    // Connect to the signal to disable call routing
-    success = connection.connect(QString(""),QString(""),
-                       APPLICATION_DBUS_INTERFACE,
-                       QString("stopOutgoingCallMonitor"),this,
-                       SLOT(stopOutgoingCallMonitor()));
-
-    if (success){
-        qDebug() << "Successfully connected to Dbus signal org.maemo.vicar.stopOutgoingCallMonitor";
-    }
-    else{
-        qDebug() << "Failed to connect to Dbus signal org.maemo.vicar.stopOutgoingCallMonitor";
-        qDebug() <<"DBus Error: "<< qPrintable(dbusUtility.getErrorMessage());
-    }
-
-}
-
-void CallRouter::disconnectFromDBusSignals(){
-
-    QDBusConnection connection = dbusUtility.getConnection();
-
-    // Disconnect from the signal to enable call routing
-    bool success = connection.disconnect(QString(""),QString(""),
-                       APPLICATION_DBUS_INTERFACE,
-                       QString("startOutgoingCallMonitor"),this,
-                       SLOT(startOutgoingCallMonitor()));
-
-    if (success){
-        qDebug() << "Successfully disconnected from Dbus signal org.maemo.vicar.startOutgoingCallMonitor";
-    }
-    else{
-        qDebug() << "Failed to disconnect from Dbus signal org.maemo.vicar.startOutgoingCallMonitor";
-        qDebug() <<"DBus Error: "<< qPrintable(dbusUtility.getErrorMessage());
-    }
-
-    // Disconnect from the signal to disable call routing
-    success = connection.connect(QString(""),QString(""),
-                       APPLICATION_DBUS_INTERFACE,
-                       QString("stopOutgoingCallMonitor"),this,
-                       SLOT(stopOutgoingCallMonitor()));
-
-    if (success){
-        qDebug() << "Successfully disconnected from Dbus signal org.maemo.vicar.stopOutgoingCallMonitor";
-    }
-    else{
-        qDebug() << "Failed to disconnect from Dbus signal org.maemo.vicar.stopOutgoingCallMonitor";
-        qDebug() <<"DBus Error: "<< qPrintable(dbusUtility.getErrorMessage());
-    }
-
-}
-
-bool CallRouter::startOutgoingCallMonitor(){
-
-    // Connect to DBus to monitor all outgoing calls
-
-    QDBusConnection connection = dbusUtility.getConnection();
-
-
-    // Declare the slot to be executed when new calls are placed
-
-    bool success = connection.connect(QString(""),
-                       CSD_CALL_PATH,
-                       CSD_CALL_INTERFACE,
-                       QString("CreateRequested"),this,
-                       SLOT(processOutgoingCall(const QDBusMessage&)));
-
-    if (success){
-        qDebug() << "Successfully connected to Dbus signal CreateRequested in interface "<< CSD_CALL_INTERFACE;
-    }
-    else{
-        qDebug() << "Failed to connect to Dbus signal CreateRequested in interface " << CSD_CALL_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
-    }
-    return success;
-}
-
-bool CallRouter::stopOutgoingCallMonitor(){
-
-    this->stopCallStatusMonitors();
-
-    //Disconnect the slots from Dbus signals
-    QDBusConnection connection = dbusUtility.getConnection();
-
-    // Disconnect the slot for new calls
-    bool status = connection.disconnect(QString(""),
-                                           CSD_CALL_PATH,
-                                           CSD_CALL_INTERFACE,
-                                           QString("CreateRequested"),this,
-                                           SLOT(processOutgoingCall(const QDBusMessage&)));
-
-    if (status){
-        qDebug() << "Successfully disconnected from Dbus signal CreateRequested in interface "<< CSD_CALL_INTERFACE;
-    }
-    else{
-        qDebug() << "Failed to disconnect from Dbus signal CreateRequested in interface "<< CSD_CALL_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
-    }
-    return status;
-}
-
-void CallRouter::processOutgoingCall(const QDBusMessage& dbusMessage){
-
-    //Verify Whether Call Routing is Enabled
-    bool isRoutingEnabled = gconfUtility->getGconfValueBoolean("routing_enabled");
-
-    if (isRoutingEnabled){
-        //User is making a phone call. Get the phone number and verify if it is an international number
-        QList<QVariant> listArguments = dbusMessage.arguments();
-        QString strInternationalNumber =  listArguments.first().toString();
-
-        qDebug() << "New Call Identified. Destination number is " << strInternationalNumber;
-
-         if (strInternationalNumber.startsWith("+") ||
-             strInternationalNumber.startsWith("00"))
-         {
-             qDebug() << "International number "<< strInternationalNumber << " recognized. Starting proceedings..";
-
-             //Check whether this is one of the excluded country codes
-             if (!isExcludedNumber(strInternationalNumber)){
-
-                //International number. Disconnect the current call (A new call will be placed)
-
-                //No arguments required to cancel the current call
-                QList<QVariant> argsToSend;
-                bool status = dbusUtility.sendMethodCall(CSD_CALL_SERVICE,
-                                                         CSD_CALL_PATH,
-                                                         CSD_CALL_INTERFACE,
-                                                         QString("Release"),argsToSend);
-
-                QString strUserMessage;
-                if (status){
-                    strUserMessage = QString("Routing international call via ").append(APPLICATION_FRIENDLY_NAME).append("..");
-                    qDebug() << strUserMessage;
-                    strLastDialedNumber = strInternationalNumber;
-                }
-                else{
-                    strUserMessage = QString("Call could not be cancelled.");
-                    qDebug() << dbusUtility.getErrorMessage();
-                }
-
-                dbusUtility.displayNotification(strUserMessage);
-
-                //Wait for a few seconds before the current call is completely disconnected
-                QTimer *timer = new QTimer(this);
-                timer->setSingleShot(true);
-                connect(timer, SIGNAL(timeout()), this, SLOT(callViaCallingCard()));
-                timer->start(3000);
-             }
-        }
-     }
-}
-
 QString CallRouter::callViaCallingCard(){
         //Now call the calling card number. This is generally a local and/or tollfree number
 
-        QString strCallingCardNumber = gconfUtility->getGconfValueString("calling_card_number");
+        QString strCallingCardNumber = d->gconfUtility->getGconfValueString("calling_card_number");
 
         qDebug() << "Initiating call to "<< strCallingCardNumber;
 
@@ -262,23 +106,22 @@ QString CallRouter::callViaCallingCard(){
         }
         else {
             strUserMessage = QString("Unable to initiate new call to ").append(strCallingCardNumber);
-            strErrorMessage = dbusUtility.getErrorMessage();
+            strErrorMessage = d->dbusUtility->getErrorMessage();
             qDebug() << strErrorMessage;
             strLastDialedNumber.clear();
         }
 
-        dbusUtility.displayNotification(strUserMessage);
+        d->dbusUtility->displayNotification(strUserMessage);
         return strErrorMessage;
 }
 
-
 bool CallRouter::placeCall(QString number){
 
     QList<QVariant> argsToSend;
     argsToSend.append(number);
     argsToSend.append(0);
 
-    bool status = dbusUtility.sendMethodCall(CSD_SERVICE,
+    bool status = d->dbusUtility->sendMethodCall(CSD_SERVICE,
                                              CSD_CALL_PATH,
                                          CSD_CALL_INTERFACE,
                                          QString("CreateWith"),argsToSend);
@@ -291,7 +134,7 @@ void CallRouter::startCallStatusMonitors(){
        We need this to confirm whether a call went though successfully.
     */
 
-    QDBusConnection connection = dbusUtility.getConnection();
+    QDBusConnection connection = d->dbusUtility->getConnection();
 
     bool success = connection.connect(QString(""),
                            CSD_CALL_INSTANCE_PATH,
@@ -304,7 +147,7 @@ void CallRouter::startCallStatusMonitors(){
     }
     else{
         qDebug() << "Failed to connect to Dbus signal AudioConnect in interface "<< CSD_CALL_INSTANCE_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
+        qDebug() <<"DBus Error: "<< d->dbusUtility->getErrorMessage();
     }
 
     /* Declare the slot to be executed when the call is terminated (due to connection errors etc).
@@ -322,7 +165,7 @@ void CallRouter::startCallStatusMonitors(){
     }
     else{
         qDebug() << "Failed to connect to Dbus signal Terminated in interface "<< CSD_CALL_INSTANCE_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
+        qDebug() <<"DBus Error: "<< d->dbusUtility->getErrorMessage();
     }
 
     /* Declare the slot to be executed when a call is received
@@ -345,7 +188,7 @@ void CallRouter::startCallStatusMonitors(){
     }
     else{
         qDebug() << "Failed to connect to Dbus signal Coming in interface" << CSD_CALL_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
+        qDebug() <<"DBus Error: "<< d->dbusUtility->getErrorMessage();
     }
 }
 
@@ -353,7 +196,7 @@ void CallRouter::stopCallStatusMonitors(){
 
     strLastDialedNumber.clear();
 
-    QDBusConnection connection = dbusUtility.getConnection();
+    QDBusConnection connection = d->dbusUtility->getConnection();
 
     // Disconnect the slot for audio connection status
     bool status = connection.disconnect(QString(""),
@@ -367,7 +210,7 @@ void CallRouter::stopCallStatusMonitors(){
     }
     else{
         qDebug() << "Failed to disconnect from Dbus signal AudioConnect in interface "<< CSD_CALL_INSTANCE_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
+        qDebug() <<"DBus Error: "<< d->dbusUtility->getErrorMessage();
     }
 
     // Disconnect the slot for monitoring terminated calls
@@ -382,7 +225,7 @@ void CallRouter::stopCallStatusMonitors(){
     }
     else{
         qDebug() << "Failed to disconnect from Dbus signal Terminated in interface "<< CSD_CALL_INSTANCE_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
+        qDebug() <<"DBus Error: "<< d->dbusUtility->getErrorMessage();
     }
 
     // Disconnect the slot for monitoring incoming calls
@@ -397,7 +240,7 @@ void CallRouter::stopCallStatusMonitors(){
     }
     else{
         qDebug() << "Failed to disconnect from Dbus signal Coming in interface" << CSD_CALL_INTERFACE;
-        qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
+        qDebug() <<"DBus Error: "<< d->dbusUtility->getErrorMessage();
     }
 }
 
@@ -418,7 +261,7 @@ void CallRouter::sendNumberAsDTMFCode(const QDBusMessage& dbusMessage){
             QList<QVariant> argsToSend;
             argsToSend.append(strDTMFCode);
 
-            bool status = dbusUtility.sendMethodCall(CSD_SERVICE,
+            bool status = d->dbusUtility->sendMethodCall(CSD_SERVICE,
                                                      CSD_CALL_PATH,
                                                  CSD_CALL_INTERFACE,
                                                  QString("SendDTMF"),argsToSend);
@@ -426,7 +269,7 @@ void CallRouter::sendNumberAsDTMFCode(const QDBusMessage& dbusMessage){
             if (status){
                 QString strMessage = strDTMFCode.append(" sent as DTMF code");
                 qDebug() << strMessage;
-                dbusUtility.displayNotification(strMessage);
+                d->dbusUtility->displayNotification(strMessage);
             }
             else{
                 qDebug() << "Unable to send DTMF code.";
@@ -458,7 +301,7 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
 
     if (!strNumber.isEmpty()){
 
-        //int intDTMFDelay = gconfUtility->getGconfValueInteger("dtmf_delay");
+        //int intDTMFDelay = d->gconfUtility->getGconfValueInteger("dtmf_delay");
 
         //if (intDTMFDelay <1 ) intDTMFDelay = 1;
 
@@ -468,13 +311,13 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
         strDTMFCode = QString("").fill('p',intDTMFDelay);        
 
         //Now check whether we need a prefix
-        QString strDTMFPrefix = gconfUtility->getGconfValueString("dtmf_prefix");
+        QString strDTMFPrefix = d->gconfUtility->getGconfValueString("dtmf_prefix");
         if (!strDTMFPrefix.isEmpty()){
             strDTMFCode = strDTMFCode.append(strDTMFPrefix);
         }
 
         //Get the format required by calling card from coniguration
-        QString qstrDTMFFormat = gconfUtility->getGconfValueString("dtmf_format");
+        QString qstrDTMFFormat = d->gconfUtility->getGconfValueString("dtmf_format");
         if (qstrDTMFFormat.isEmpty()) qstrDTMFFormat = "<Country Code><Area Code><Phone Number>";
 
         /* Replace 00 (international dialing code) at the beginning
@@ -499,7 +342,7 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
         strDTMFCode = strDTMFCode.append(strNumber);
 
         //Now check whether we need a suffix
-        QString strDTMFSuffix = gconfUtility->getGconfValueString("dtmf_suffix");
+        QString strDTMFSuffix = d->gconfUtility->getGconfValueString("dtmf_suffix");
         if (!strDTMFSuffix.isEmpty() && !strDTMFSuffix.contains("--None--")){
             strDTMFCode = strDTMFCode.append(strDTMFSuffix);
         }
@@ -513,7 +356,7 @@ bool CallRouter::isExcludedNumber(QString strInternationalNumber){
     bool isExcluded = false;
 
     //Get the list of excluded codes
-    QString qstrExcludedNumbers = gconfUtility->getGconfValueString("numbers_to_exclude");
+    QString qstrExcludedNumbers = d->gconfUtility->getGconfValueString("numbers_to_exclude");
     QStringList strExcludedCodeList = qstrExcludedNumbers.split(",");
     QStringListIterator iterator(strExcludedCodeList);
 
@@ -534,7 +377,7 @@ bool CallRouter::isExcludedNumber(QString strInternationalNumber){
 bool CallRouter::isRunning(){
 
     //Verify Whether Call Routing is Enabled
-    bool isRoutingEnabled = gconfUtility->getGconfValueBoolean("routing_enabled");
+    bool isRoutingEnabled = d->gconfUtility->getGconfValueBoolean("routing_enabled");
     if (isRoutingEnabled){
         return true;
     }
@@ -546,7 +389,7 @@ bool CallRouter::isRunning(){
 //DBus Method used by external applications to call via VICaR
 QString CallRouter::callInternationalNumber(const QString& strInternationalNumber){
 
-    //QDBusConnection connection = dbusUtility.getConnection();
+    //QDBusConnection connection = d->dbusUtility->getConnection();
     QString strReturnMessage = "";
 
     qDebug() << "New call requested by external application. Destination number is " << strInternationalNumber;
@@ -569,15 +412,12 @@ QString CallRouter::callInternationalNumber(const QString& strInternationalNumbe
              qDebug() << "Country code is excluded, placing call directly";
              bool status = this->placeCall(strInternationalNumber);
              if (!status){
-                 strReturnMessage = dbusUtility.getErrorMessage();
+                 strReturnMessage = d->dbusUtility->getErrorMessage();
              }
          }
     }
     else{
         strReturnMessage = "VICaR Error: Phone number is not in international format. Use +123456789 format.";
-        //QDBusMessage dbusError = QDBusMessage();
-        //dbusError = dbusError.createErrorReply("org.maemo.vicar.InvalidNumberFormat",strErrorMessage);
-        //connection.send(dbusError);
     }
 
     qDebug() << strReturnMessage;
