@@ -9,6 +9,7 @@
 
 #include <QObject>
 #include <QDBusMessage>
+#include <dbusutility.h>
 #include <gconfutility.h>
 #include "vicardbusadaptor.h"
 
@@ -27,6 +28,8 @@
 #define CSD_CALL_SERVICE	"com.nokia.csd.Call"
 #define CSD_SERVICE             "com.nokia.csd"
 
+class CallRouterPrivate;
+
 class CallRouter : public QObject
 {
 Q_OBJECT
@@ -41,27 +44,23 @@ public:
 protected:
     void startCallStatusMonitors();
     QString convertToDTMFCode(QString);
-    void connectToDBusSignals();
-    void disconnectFromDBusSignals();
+//    void connectToDBusSignals();
+//    void disconnectFromDBusSignals();
     bool isExcludedNumber(QString);
     bool placeCall(QString);
 
 protected slots:
-    void processOutgoingCall(const QDBusMessage& dbusMessage);
     QString callViaCallingCard();
     void sendNumberAsDTMFCode(const QDBusMessage& dbusMessage);
     void stopCallStatusMonitors();
 
     //D-Bus Interface Methods
 public slots:
-    bool startOutgoingCallMonitor();
-    bool stopOutgoingCallMonitor();
     bool isRunning();
     QString callInternationalNumber(const QString& internationalNumber);
 
 private:
-    GConfUtility *gconfUtility;
-    VicarDbusAdaptor *dbusAdaptor;
+    CallRouterPrivate * const d;
 };
 
 #endif // CALLROUTER_H
