@@ -14,13 +14,13 @@
 #define NOTIFICATION_PATH       "/org/freedesktop/Notifications"
 #define NOTIFICATION_INTERFACE  "org.freedesktop.Notifications"
 
-class DbusUtility
+class DbusUtility: public QObject
 {
+ Q_OBJECT
 protected:
     QDBusConnection connection;
 public:
-    DbusUtility():connection(QDBusConnection::systemBus())
-    {}
+    DbusUtility(QObject * parent = 0);
     ~DbusUtility();
     QDBusConnection getConnection();
     void setConnection(QDBusConnection connection);
